using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using VSCom.CanApi;

namespace VSCom.J1939Api
{
    class Test
    {
        static byte AddressCallback()
        {
            return 252;
        }

        public static string ByteArrayToString(byte[] ba)
        {
            string hex = BitConverter.ToString(ba);
            return hex.Replace("-", "");
        }

        static void PrintFrame(VSCAN_J1939.VSCAN_J1939_MSG msg)
        {
            Console.WriteLine(String.Format("S:{0},D:{1},P:{2},PGN:{3},L:{4},{5}",
                msg.Src, msg.Dst, msg.Priority, msg.PGN, msg.Size, ByteArrayToString(msg.Data)));
        }

        static void Main(string[] args)
        {
            VSCAN CanDevice = new VSCAN();
            VSCAN_J1939 j1939 = new VSCAN_J1939();
            VSCAN_J1939.VSCAN_J1939_ADDR_CALLBACK addrCallback = new VSCAN_J1939.VSCAN_J1939_ADDR_CALLBACK(AddressCallback);

            try
            {
                // set debugging options
                CanDevice.SetDebug(VSCAN.VSCAN_DEBUG_NONE);
                CanDevice.SetDebugMode(VSCAN.VSCAN_DEBUG_MODE_FILE);

                CanDevice.Open("COM51", VSCAN.VSCAN_MODE_NORMAL);

                CanDevice.SetSpeed(VSCAN.VSCAN_SPEED_250K);
                VSCAN_CODE_MASK codeMask = new VSCAN_CODE_MASK();
                codeMask.Code = 0;
                codeMask.Mask = 0xffffffff;
                CanDevice.SetACCCodeMask(codeMask);

                VSCAN_J1939.VSCAN_J1939_NAME name = new VSCAN_J1939.VSCAN_J1939_NAME();
                name.ArbAddrCapable = 0;
	            name.IndustryGroup = 0x01;
	            name.VehicleSys = 0x01;
	            name.Func = 0xff;
	            name.ManCode = 0x66;

                j1939.Init(CanDevice.Handle, name, addrCallback);

                VSCAN_J1939.VSCAN_J1939_MSG[] msg = new VSCAN_J1939.VSCAN_J1939_MSG[1];
                /*msg[0].Dst = 255;
	            msg[0].Priority = 0x5;
	            msg[0].PGN = 65262;
                msg[0].Data = new byte[] { 0x10, 0x20, 0x30, 0x40, 0x50, 0x60, 0x70, 0x80 };
	            msg[0].Size = (UInt16)msg[0].Data.Length;
	            msg[0].Status = VSCAN_J1939.VSCAN_J1939_STATUS_FILL_IN;
                UInt32 written = 0;
                j1939.Write(msg, 1, ref written);*/

                UInt32 read = 0;
                for (;;)
                {
                    if (j1939.Read_BufferCount() > 0)
                    {
                        j1939.Read(ref msg, 1, ref read);
                        PrintFrame(msg[0]);
                    }
                }

                j1939.Free();
                CanDevice.Close();
            }
            catch(Exception ex)
            {
                Console.WriteLine("Error: " + ex.Message);
            }
        }
    }
}
