module.exports = function(RED) {
	var ffi = require("ffi")
	var ref = require("ref")
	var vscan = require("node-red-contrib-vscan-wrapper")
	
    function VsCanNodeRead(config) {
        RED.nodes.createNode(this,config);
        var node = this;
		var log = RED.log;
		var timerID = null;
		var canHandle = 0;
		
		// FIXME:
		// We did not receive any frames, if we re-open the channel (tty).
		// It did work with our node.js test program?! Changes in further
		// channel configs will be dismissed without node-red restart.
		if (typeof global.canHandle !== 'undefined')
			canHandle = global.canHandle;

		// switch on debugging on console
		//vscan.VSCAN_Ioctl(0, vscan.VSCAN_IOCTL_SET_DEBUG, vscan.VSCAN_DEBUG_HIGH);
		//vscan.VSCAN_Ioctl(0, vscan.VSCAN_IOCTL_SET_DEBUG_MODE, vscan.VSCAN_DEBUG_MODE_CONSOLE); 

		if (timerID == null) {
			log.debug(`[VSCAN] Start interval timer with ${config.interval} ms`);
			timerID = setInterval(j1939PollingRead, config.interval);
		}
	
		function j1939OpenChannel() {
			log.debug(`[VSCAN] Open VSCAN channel ${config.device}`);
			canHandle = vscan.VSCAN_Open(config.device, 0);
			if (canHandle <= 0)
			{
				log.error(`[VSCAN] Open VSCAN channel ${config.device} failed with error ${canHandle}`);
				return canHandle;
			}
			var codeMask = new vscan.vsCanCodeMask;
			codeMask.Code = 0;
			codeMask.Mask = 0xffffffff;
			log.debug(`[VSCAN] Set VSCAN channel code ${codeMask.Code.toString(16)} and accaptence mask ${codeMask.Mask.toString(16)}`);
			status = vscan.VSCAN_Ioctl(canHandle, vscan.VSCAN_IOCTL_SET_ACC_CODE_MASK, codeMask.ref());
			log.debug(`[VSCAN] Set VSCAN channel speed to 250kbit/s`);
			status = vscan.VSCAN_Ioctl(canHandle, vscan.VSCAN_IOCTL_SET_SPEED, vscan.VSCAN_SPEED_250K);
			
			var name = new vscan.vsJ1939Name;
			name.ArbAddrCapable = 0;
			name.IndustryGroup = 0x01;
			name.VehicleSys = 0x01;
			name.Func = 0xff;
			name.ManCode = 0x66;

			var addrCallback = ffi.Callback(vscan.vsJ1939Addr, [],
			  function() {
				return 252;
			  });
			// make an extra reference to the callback pointer to avoid GC
			process.on('exit', function() {
			  addrCallback
			});

			var version = vscan.VSCAN_J1939_ApiVersion();
			log.info(`[VSCAN] J1939 Version: ${version.Major}.${version.Minor}.${version.SubMinor}`);
			log.debug(`[VSCAN] Initializing J1939 stack`);
			status = vscan.VSCAN_J1939_Init(canHandle, name, addrCallback);
			
			return canHandle;
		}
		
		function j1939PollingRead() {
			var node_msg;
			var status;
			var msg = new vscan.vsJ1939Msg;
			var read = ref.alloc('uint32'); 
			
			if (canHandle <= 0) {
				canHandle = j1939OpenChannel();
				if (canHandle <= 0)
					return;
				global.canHandle = canHandle;
			}
			while ((cnt = vscan.VSCAN_J1939_Read_BufferCount(canHandle)) > 0) {
				if (vscan.VSCAN_J1939_Read(canHandle, msg.ref(), 1, read) < 0) {
					log.error("[VSCAN] VSCAN_J1939_Read failed");
				}
				node_msg = { payload: msg };
				node.send(node_msg);
			}
		}
	
		node.on('close', function() {
			clearInterval(timerID); // clear Timer from events
			timerID = null;
			// see comment on init above
			/*if (canHandle > 0)
			{
				log.debug(`[VSCAN] Closing J1939 stack and VSCAN handle ${canHandle}`);
				vscan.VSCAN_J1939_Free(canHandle); 
				vscan.VSCAN_Close(canHandle);
				canHandle = 0;
			}*/
		});
	}
    RED.nodes.registerType("vscan",VsCanNodeRead);
}
